#include <msp/gl/extensions/arb_enhanced_layouts.h>
#include <msp/gl/extensions/arb_explicit_attrib_location.h>
#include <msp/gl/extensions/arb_explicit_uniform_location.h>
#include <msp/gl/extensions/arb_gpu_shader5.h>
#include <msp/gl/extensions/arb_separate_shader_objects.h>
#include <msp/gl/extensions/arb_uniform_buffer_object.h>
#include <msp/gl/extensions/ext_gpu_shader4.h>
#include <msp/gl/extensions/ext_texture_array.h>
#include <msp/gl/misc.h>
#include "features.h"

namespace Msp {
namespace GL {
namespace SL {

Features::Features():
	gl_api(OPENGL),
	arb_enhanced_layouts(false),
	arb_explicit_attrib_location(false),
	arb_explicit_uniform_location(false),
	arb_gpu_shader5(false),
	arb_separate_shader_objects(false),
	arb_uniform_buffer_object(false),
	ext_gpu_shader4(false),
	ext_texture_array(false),
	constant_id_range(0x80000000U),
	uniform_binding_range(84),
	texture_binding_range(96)
{ }

Features Features::from_context()
{
	Features features;
	features.gl_api = get_gl_api();
	features.glsl_version = get_glsl_version();
	features.arb_enhanced_layouts = ARB_enhanced_layouts;
	features.arb_explicit_attrib_location = ARB_explicit_attrib_location;
	features.arb_explicit_uniform_location = ARB_explicit_uniform_location;
	features.arb_gpu_shader5 = ARB_gpu_shader5;
	features.arb_separate_shader_objects = ARB_separate_shader_objects;
	features.arb_uniform_buffer_object = ARB_uniform_buffer_object;
	features.ext_gpu_shader4 = EXT_gpu_shader4;
	features.ext_texture_array = EXT_texture_array;
	features.uniform_binding_range = get_i(GL_MAX_UNIFORM_BUFFER_BINDINGS);
	features.texture_binding_range = get_i(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS);
	return features;
}

Features Features::from_version(const Version &ver)
{
	Features features;
	features.gl_api = OPENGL;
	features.glsl_version = ver;
	features.arb_enhanced_layouts = (ver>=Version(4, 40));
	features.arb_explicit_attrib_location = (ver>=Version(1, 30));
	features.arb_explicit_uniform_location = (ver>=Version(4, 30));
	features.arb_gpu_shader5 = (ver>=Version(4, 0));
	features.arb_separate_shader_objects = (ver>=Version(4, 10));
	features.arb_uniform_buffer_object = (ver>=Version(1, 50));
	features.ext_gpu_shader4 = (ver>=Version(1, 20));
	features.ext_texture_array = (ver>=Version(1, 30));
	features.uniform_binding_range = (ver>=Version(4, 30) ? 84 : ver>=Version(4, 0) ? 60 :
		ver>=Version(3, 30) ? 36 : 24);
	features.texture_binding_range = (ver>=Version(4, 30) ? 96 : ver>=Version(4, 0) ? 80 :
		ver>=Version(1, 50) ? 48 : ver>=Version(1, 40) ? 32 : 16);
	return features;
}

Features Features::latest()
{
	return from_version(Version(4, 60));
}

} // namespace SL
} // namespace GL
} // namespace Msp
