#ifndef MSP_GL_SL_REFLECT_H_
#define MSP_GL_SL_REFLECT_H_

#include "visitor.h"

namespace Msp {
namespace GL {
namespace SL {

/** Determines the number of interface locations required by a variable. */
class LocationCounter: private NodeVisitor
{
private:
	unsigned r_count;

public:
	LocationCounter();

	unsigned apply(VariableDeclaration &v) { v.visit(*this); return r_count; }

private:
	virtual void visit(BasicTypeDeclaration &);
	virtual void visit(ImageTypeDeclaration &);
	virtual void visit(StructDeclaration &);
	virtual void visit(VariableDeclaration &);
};

/** Determines the size and alignment of a variable, in bytes. */
class MemoryRequirementsCalculator: private NodeVisitor
{
public:
	struct Result
	{
		unsigned size;
		unsigned alignment;

		Result(unsigned s, unsigned a): size(s), alignment(a) { }
	};
private:
	unsigned r_size;
	unsigned r_alignment;
	int r_offset;

public:
	Result apply(VariableDeclaration &v) { v.visit(*this); return Result(r_size, r_alignment); }

private:
	virtual void visit(BasicTypeDeclaration &);
	virtual void visit(StructDeclaration &);
	virtual void visit(VariableDeclaration &);
};

/** Collects dependencies of a function.  This includes global variables,
interface blocks, other functions and types. */
class DependencyCollector: private TraversingVisitor
{
private:
	std::set<Node *> dependencies;
	std::set<Node *> locals;

public:
	std::set<Node *> apply(FunctionDeclaration &);

private:
	virtual void visit(VariableReference &);
	virtual void visit(InterfaceBlockReference &);
	virtual void visit(FunctionCall &);
	virtual void visit(VariableDeclaration &);
};

} // namespace SL
} // namespace GL
} // namespace Msp

#endif
